/*==============================================================================
		Common Functions for Simple Melter & Simple Freezer
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "Common.h"
//------------------------------------------------------------------------------
// BeOS Header
#include <Directory.h>
#include <FindDirectory.h>
#include <Path.h>
// C Header
//------------------------------------------------------------------------------

/*=== Memo =====================================================================
==============================================================================*/

//==============================================================================
void	load_trans_archiver(vector<TTransArchiver*> *oArcvers)
{
	char			aAddOnFName[B_FILE_NAME_LENGTH];				// Filename of Addon
	image_id		aImgID;
	string			aMsgStr;
	BDirectory		aSearchFolder;									// Folder that has Addons
	BEntry			aAddOnEntry;									// Entry of Addon
	BPath			aAddOnPath;										// Fullpath of Addon
	BTranslator*	aTranslator;
	BTranslator*	(*aMakeTransFunc)(int32,image_id,uint32, ...);	// Translator making func
	TTransArchiver*	aTransArchiver;									// Trans Archiver

	if ( find_directory(B_COMMON_ADDONS_DIRECTORY,&aAddOnPath) != B_OK )
		return;
	aAddOnPath.Append(TRANSLATOR_ADDON_PATH);
	aSearchFolder.SetTo(aAddOnPath.Path());
	// Search TransArchiver in /boot/home/config/add-ons/Translators
	while ( aSearchFolder.GetNextEntry(&aAddOnEntry,true) == B_OK ) {
		aAddOnEntry.GetName(aAddOnFName);
		// Name check
		if ( strncasecmp(aAddOnFName,TRANS_ARCHIVER_FNAME_PREFIX,strlen(TRANS_ARCHIVER_FNAME_PREFIX)) != 0 )	// TransArchiver's filename must begin at "Trans"
			continue;
		aAddOnEntry.GetPath(&aAddOnPath);
		// Load
		if ( (aImgID = load_add_on(aAddOnPath.Path())) < 0 )
			continue;
		// Search making function
		if ( get_image_symbol(aImgID,"make_nth_translator",B_SYMBOL_TYPE_TEXT,(void **)&aMakeTransFunc) != B_OK )
			continue;
		// Get TransArchiver
		aTranslator = (*aMakeTransFunc)(0,aImgID,0);
		aTransArchiver = dynamic_cast<TTransArchiver *>(aTranslator);
		if ( aTransArchiver == NULL ) {	// It's not TransArchiver, only a Translator
			aTranslator->Release();
			continue;
		}
		// Compatibility check (Recommended)
		if ( aTransArchiver->AppCompatiIdent() != aTransArchiver->SuperClassCompatiIdent()
				|| aTransArchiver->AppCompatiIdent() != aTransArchiver->SubClassCompatiIdent() ) {
			cout << "Compatibility Error: " << aAddOnFName << endl;
			aTranslator->Release();
			continue;
		}
		// Found
		oArcvers->push_back(aTransArchiver);
		cout << "Loaded: " << aTransArchiver->TranslatorName() << " (" << aAddOnFName << ")" << endl;
	}
	if ( oArcvers->empty() ) {
		cout << "No TransArchiver found." << endl;
	}
	cout << endl;
}
//------------------------------------------------------------------------------
//==============================================================================
