#include <InterfaceDefs.h>

/*============================================================================*/
const char	*gAppName = "Simple Freezer   * Sample Code for Freezing *";
const char	*gAppVer = "Ver 1.00";
const char	*gCright = "Copyright "B_UTF8_COPYRIGHT" 2000 by SHINTA";
const char	*gAppSignature = "application/x-vnd.SHINTA-SimpleFreezer";
/*============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2000.12.29.Fr 開発着手。
 1.00 | 2000.12.29.Fr オリジナルバージョン。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "Freezer.h"
//------------------------------------------------------------------------------
// BeOS Header
#include <Directory.h>
#include <Path.h>
//------------------------------------------------------------------------------

/*=== Memo =====================================================================
==============================================================================*/

//==============================================================================
int		main(int oArgC,char *oArgV[])
{
	string						aFreezeFolder;	// Foldername having files to be frozen (full path)
	vector<TTransArchiver *>	aArcvers;		// Loaded Trans Archivers
	TTransArchiver*				aFreezer;		// Freezer

	exp_app_info();
	if ( !get_freeze_folder_name(oArgC,oArgV,&aFreezeFolder) )
		return 1;
	load_trans_archiver(&aArcvers);
	aFreezer = find_freezer(aArcvers);
	if ( aFreezer == NULL )
		return 1;
	freeze(aFreezeFolder,aFreezer);
	return 0;
}
//==============================================================================
void	exp_app_info()
{
	cout << endl << gAppName << "    " << gAppVer << endl;
	cout << gCright << endl << endl;
	cout << "Make archive \"" << DEST_ARC_FNAME << "\"" << endl << endl;
}
//------------------------------------------------------------------------------
TTransArchiver*	find_freezer(const vector<TTransArchiver*> &oArcvers)
{
	int32						aOutCount;
	const translation_format*	aOutFmts;
	vector<TTransArchiver *>::const_iterator	p;
	TTransArchiver*				aFreezer = NULL;

	// Find
	p = oArcvers.begin();
	while ( p != oArcvers.end() ) {
		aOutFmts = (*p)->OutputFormats(&aOutCount);
		for ( int i = 0 ; i < aOutCount ; i++ ) {
			if ( aOutFmts[i].group != B_TRANSLATOR_BITMAP && aOutFmts[i].quality != 0
					&& aOutFmts[i].capability != 0 ) {
				aFreezer = (*p);
				break;
			}
		}
		if ( aFreezer != NULL )
			break;
		p++;
	}
	// Show Found TransArchiver
	if ( aFreezer != NULL ) {
		cout << "Uses: " << aFreezer->TranslatorName() << endl << endl;
	} else {
		cout << "No TransArchiver can freeze the source folder." << endl << endl;
	}
	return aFreezer;
}
//------------------------------------------------------------------------------
void	freeze(string oFolderName,TTransArchiver *oFreezer)
{
	char*				aFreezeFileName;
	entry_ref			aArcERef;
	entry_ref			aFreezeFileERef;
	list<const char *>	aFreezeFiles;
	list<const char *>::iterator	p;
	BDirectory			aFreezeFolder(oFolderName.c_str());

	// SetTo
	get_ref_for_path(DEST_ARC_FNAME,&aArcERef);
	oFreezer->SetTo(&aArcERef,B_READ_WRITE|B_CREATE_FILE|B_ERASE_FILE);
	// Make Freeze File List - All Files in the Folder
	while ( aFreezeFolder.GetNextRef(&aFreezeFileERef) == B_OK ) {
		aFreezeFileName = new char [B_FILE_NAME_LENGTH];
		strcpy(aFreezeFileName,BPath(&aFreezeFileERef).Path());
		aFreezeFiles.push_back(aFreezeFileName);
		cout << "Freeze: " << aFreezeFileName << endl;
	}
	cout << endl;
	// Freeze
	oFreezer->Freeze(aFreezeFiles);
	// Release Freeze File List
	p = aFreezeFiles.begin();
	while ( p != aFreezeFiles.end() ) {
		delete [] (*p);
		p++;
	}
	aFreezeFiles.clear();
}
//------------------------------------------------------------------------------
bool	get_freeze_folder_name(int oArgC,char *oArgV[],string *oFolderName)
{
	BEntry	aEntry;

	try {
		if ( oArgC <= 1 )
			throw 0;
		if ( oArgV[1][0] != '/' )
			throw 0;
		if ( aEntry.SetTo(oArgV[1]) != B_OK )
			throw 0;
		if ( !aEntry.Exists() )
			throw 0;
		if ( !aEntry.IsDirectory() )
			throw 0;
		*oFolderName = oArgV[1];
		cout << "Source folder: " << oArgV[1] << endl << endl;
		return true;
	} catch (...) {
		cout << "Usage:" << endl << "$ Freezer.exe <FolderName>" << endl
				<< "Folder(fullpath) must exist" << endl << endl;
		return false;
	}
}
//------------------------------------------------------------------------------
