#include <InterfaceDefs.h>

/*============================================================================*/
const char	*gAppName = "Simple Melter   * Sample Code for Melting *";
const char	*gAppVer = "Ver 1.00";
const char	*gCright = "Copyright "B_UTF8_COPYRIGHT" 2000 by SHINTA";
const char	*gAppSignature = "application/x-vnd.SHINTA-SimpleMelter";
/*============================================================================*/

/*==============================================================================
 Ver. |					主な改良点
--------------------------------------------------------------------------------
 ---- | 2000.12.28.Th 開発着手。
 1.00 | 2000.12.29.Fr オリジナルバージョン。
==============================================================================*/

/*=== Memo =====================================================================
==============================================================================*/

//------------------------------------------------------------------------------
#include "Melter.h"
//------------------------------------------------------------------------------
// BeOS Header
#include <Message.h>
#include <NodeInfo.h>
// C Header
//------------------------------------------------------------------------------

/*=== Memo =====================================================================
==============================================================================*/

//==============================================================================
int		main(int oArgC,char *oArgV[])
{
	string						aFPMeltFName;	// Filename to be melt (full path)
	vector<TTransArchiver *>	aArcvers;		// Loaded Trans Archivers
	TTransArchiver*				aMelter;		// Melter

	exp_app_info();
	if ( !get_melt_file_name(oArgC,oArgV,&aFPMeltFName) )
		return 1;
	load_trans_archiver(&aArcvers);
	aMelter = find_melter(aFPMeltFName,aArcvers);
	if ( aMelter == NULL )
		return 1;
	melt(aFPMeltFName,aMelter);
	return 0;
}
//==============================================================================
void	exp_app_info()
{
	cout << endl << gAppName << "    " << gAppVer << endl;
	cout << gCright << endl << endl;
	cout << "Melt archives to " << MELT_FOLDER << endl << endl;
}
//------------------------------------------------------------------------------
TTransArchiver*	find_melter(string oFName,const vector<TTransArchiver*> &oArcvers)
{
	float				aMaxCapability = 0;
	uint32				aOutType;
	status_t			aResult;
	translation_format	aInFmt;
	translator_info		aOutInfo;
	vector<TTransArchiver *>::const_iterator	p;
	BFile				aFile(oFName.c_str(),B_READ_ONLY);
	BMessage			aIoExtension;
	BNodeInfo			aFileInfo;
	TTransArchiver*		aMelter = NULL;

	// Get Info of Target Archive
	aFileInfo.SetTo(&aFile);
	aFileInfo.GetType(aInFmt.MIME);
	// Set Info of Output
	aOutType = TRANS_ARCHIVER_RAW_TYPE;
	// Find
	p = oArcvers.begin();
	while ( p != oArcvers.end() ) {
		aResult = (*p)->Identify(&aFile,&aInFmt,&aIoExtension,&aOutInfo,aOutType);
		if ( aResult == B_OK ) {	// Can Treat
			if ( aMelter == NULL ) {
				aMelter = (*p);
				aMaxCapability = aOutInfo.capability;
			} else {
				if ( aOutInfo.capability > aMaxCapability ) {	// Better
					aMelter = (*p);
					aMaxCapability = aOutInfo.capability;
				}
			}
		}
		p++;
	}
	// Show Found TransArchiver
	if ( aMelter != NULL ) {
		cout << "Uses: " << aMelter->TranslatorName() << endl << endl;
	} else {
		cout << "No TransArchiver can melt the target archive." << endl << endl;
	}
	return aMelter;
}
//------------------------------------------------------------------------------
bool	get_melt_file_name(int oArgC,char *oArgV[],string *oFName)
{
	BEntry	aEntry;

	try {
		if ( oArgC <= 1 )
			throw 0;
		if ( oArgV[1][0] != '/' )
			throw 0;
		if ( aEntry.SetTo(oArgV[1]) != B_OK )
			throw 0;
		if ( !aEntry.Exists() )
			throw 0;
		*oFName = oArgV[1];
		cout << "Target archive: " << oArgV[1] << endl << endl;
		return true;
	} catch (...) {
		cout << "Usage:" << endl << "$ Melter.exe <ArchiveFileName>" << endl
				<< "ArchiveFile(fullpath) must exist" << endl << endl;
		return false;
	}
}
//------------------------------------------------------------------------------
void	melt(string oFPMeltFName,TTransArchiver *oMelter)
{
	char					aMTime[64];
	entry_ref				aMeltRef;
	list<TIndividualInfo>	aInfos;
	list<TIndividualInfo>::iterator	p;

	// Set Melter to Target Archive
	get_ref_for_path(oFPMeltFName.c_str(),&aMeltRef);
	oMelter->SetTo(&aMeltRef,B_READ_ONLY);
	// Show Contents
	oMelter->GetAllIndividualInfos(&aInfos);
	cout << "<Name>, <FrozenSize>, <OriginalSize>, <Ratio>, <ModificationTime>, <Format>, <CRC>" << endl;
	p = aInfos.begin();
	while ( p != aInfos.end() ) {
		oMelter->FormatTime(aMTime,p->mModificationTime,"yyyy/mm/dd");
		cout << p->mFileName << ", " << p->mFrozenSize << ", " << p->mOriginalSize << ", "
				<< p->Ratio()/10 << "%, " << aMTime << ", " << p->mFormat << ", " << hex << p->mCRC << dec << endl;
		p++;
	}
	cout << endl;
	// Melt
	oMelter->MeltAll(MELT_FOLDER);
}
//==============================================================================
