/*==============================================================================
各種トランスレーションアーカイバの仮想基底クラス
==============================================================================*/

/*== MEMO ======================================================================
削除
リネーム
ログ
アドオン側は、実行時にライブラリが見つからなくても大丈夫。
==============================================================================*/

#include "TransArchiver.h"
//------------------------------------------------------------------------------
#include <Locker.h>
#include <add/SHIDebug.h>
//------------------------------------------------------------------------------
static BLocker	sgTimeLocker;	// Time系関数を使うときのロック
//==============================================================================
TTransArchiver::TTransArchiver()
		: SUPER()
{
	mJunkPathsInFreezing = false;
	mJunkPathsInMelting = false;
}
//------------------------------------------------------------------------------
TTransArchiver::~TTransArchiver()
{
}
//------------------------------------------------------------------------------
void	TTransArchiver::FormatTime(char *oDest,bigtime_t oTimeUs,const char *oFormat) const
{
	int			aYear;
	time_t		aTimeS = oTimeUs/(1000*1000);	//μ秒→秒
	struct tm	aTM;
	BString		aFmtTime = oFormat;
	BString		aTmpString;

//	DBEXP("befor cvt - TTransArchiver::FormatTime",aTimeMs);
	sgTimeLocker.Lock();	//gmtime()が指すデータはスレッドセーフでないため
	aTM = *gmtime(&aTimeS);
	sgTimeLocker.Unlock();
	//年
	aYear = aTM.tm_year+1900;
	if ( aYear < TRANS_ARCHIVER_YEAR_BORDER ) {
		aYear += 100;
	}
	aTmpString = BString() << aYear;
	aFmtTime.IReplaceAll("yyyy",aTmpString.String());
	//月
	FormatTimeReplace(&aFmtTime,"mm",aTM.tm_mon+1);
	//日
	FormatTimeReplace(&aFmtTime,"dd",aTM.tm_mday);
	//時
	FormatTimeReplace(&aFmtTime,"hh",aTM.tm_hour);
	//分
	FormatTimeReplace(&aFmtTime,"nn",aTM.tm_min);
	//秒
	FormatTimeReplace(&aFmtTime,"ss",aTM.tm_sec);
	strcpy(oDest,aFmtTime.String());
}
//------------------------------------------------------------------------------
void	TTransArchiver::FormatTimeReplace(BString *oString,const char *oOld,int oNew) const
{
	BString		aNewString;

	aNewString << oNew;
	if ( aNewString.Length() == 1 ) {
		aNewString.Prepend("0");
	}
	oString->IReplaceAll(oOld,aNewString.String());
}
//------------------------------------------------------------------------------
status_t	TTransArchiver::Freeze(const list<const char *> &oFreezeFiles,const char *oBaseFolder)
/*	''''''''''''''''''''''''''''''
Opt:
	oFreezeFiles中のファイル名は、絶対または相対パス
	oBaseFolderは、NULLまたは絶対パス。'/' で終わっても終わらなくても可
,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,	*/
{
	return B_UNSUPPORTED;
}
//------------------------------------------------------------------------------
status_t	TTransArchiver::GetAllIndividualInfos(list<TIndividualInfo> *oInfos)
{
	return B_UNSUPPORTED;
}
//------------------------------------------------------------------------------
status_t	TTransArchiver::Identify(BPositionIO *oInSource,const translation_format *oInFormat
		,BMessage *oIoExtension,translator_info *oOutInfo,uint32 oOutType)
{
	return B_NO_TRANSLATOR;
}
//------------------------------------------------------------------------------
const translation_format*	TTransArchiver::InputFormats(int32 *oOutCount) const
{
	*oOutCount = 0;
	return NULL;
}
//------------------------------------------------------------------------------
bool	TTransArchiver::JunkPathsInFreezing() const
{
	return mJunkPathsInFreezing;
}
//------------------------------------------------------------------------------
bool	TTransArchiver::JunkPathsInMelting() const
{
	return mJunkPathsInMelting;
}
//------------------------------------------------------------------------------
status_t	TTransArchiver::Melt(const list<const char *> &oMeltFiles,const char *oDestFolder)
{
	return B_UNSUPPORTED;
}
//------------------------------------------------------------------------------
status_t	TTransArchiver::MeltAll(const char *oDestFolder)
{
	return B_UNSUPPORTED;
}
//------------------------------------------------------------------------------
const translation_format*	TTransArchiver::OutputFormats(int32 *oOutCount) const
{
	*oOutCount = 0;
	return NULL;
}
//------------------------------------------------------------------------------
void	TTransArchiver::SetJunkPathsInFreezing(bool oJunk)
{
	mJunkPathsInFreezing = oJunk;
}
//------------------------------------------------------------------------------
void	TTransArchiver::SetJunkPathsInMelting(bool oJunk)
{
	mJunkPathsInMelting = oJunk;
}
//------------------------------------------------------------------------------
status_t	TTransArchiver::SetTo(const entry_ref *oArcRef,uint32 oOpenMode)
{
	return mArcFile.SetTo(oArcRef,oOpenMode);
}
//------------------------------------------------------------------------------
int32	TTransArchiver::SuperClassCompatiIdent() const
{
	return COMPATIBILITY_IDENT;
}
//------------------------------------------------------------------------------
status_t	TTransArchiver::Translate(BPositionIO *oInSource,const translator_info *oInInfo,BMessage *oIoExtension
		,uint32 oOutType,BPositionIO *oOutDestination)
{
	return B_NO_TRANSLATOR;
}
//------------------------------------------------------------------------------
const char*	TTransArchiver::TranslatorInfo() const
{
	return "Trans Archiver";
}
//------------------------------------------------------------------------------
const char*	TTransArchiver::TranslatorName() const
{
	return "Trans Archiver";
}
//------------------------------------------------------------------------------
int32	TTransArchiver::TranslatorVersion() const
{
	return 100;
}
//------------------------------------------------------------------------------
void	TTransArchiver::Unset()
{
	mArcFile.Unset();
}
//==============================================================================
