#ifndef	___TransArchiver_H_
#define	___TransArchiver_H_
//------------------------------------------------------------------------------
// BeOS Header
#include <BeBuild.h>
#include <Entry.h>
#include <File.h>
#include <String.h>
#include <Translator.h>
// C++ Header
#include <list>
#include <string>
// Project Header
#include "VPTransArchiver.h"
//------------------------------------------------------------------------------
using namespace	std;
//------------------------------------------------------------------------------
#ifdef	BUILDING_TRANS_ARCHIVER
#define	TRANS_ARCHIVER_SYMBOL	_EXPORT
#else
#define TRANS_ARCHIVER_SYMBOL	_IMPORT
#endif
//------------------------------------------------------------------------------
const int		TRANS_ARCHIVER_ERR_BASE = B_ERRORS_END+0x10001;		// エラーコードの始まり
const int		TRANS_ARCHIVER_YEAR_BORDER = 70;					// 1900年代か2000年代か
const uint32	TRANS_ARCHIVER_RAW_TYPE = 'rawt';					// バイナリ
const int32		COMPATIBILITY_IDENT = 1;							// 互換性チェック用
//==============================================================================
class TRANS_ARCHIVER_SYMBOL	TTransArchiver : public BTranslator
{
public:
	// 簡易互換性チェック
	inline int32	AppCompatiIdent() const { return COMPATIBILITY_IDENT; };	// アプリケーション側にリンク
	int32			SuperClassCompatiIdent() const;								// 基底クラスＤＬＬにリンク
	virtual int32	SubClassCompatiIdent() const = 0;							// 派生クラスＤＬＬにリンク

	// 主要関数
	virtual status_t	Freeze(const list<const char *> &oFreezeFiles,const char *oBaseFolder = NULL);	//書庫にファイルを追加
	virtual status_t	GetAllIndividualInfos(list<TIndividualInfo> *oInfos);		//書庫中の全てのファイルの情報をコンテナに格納
	virtual status_t	Melt(const list<const char *> &oMeltFiles,const char *oDestFolder);		//指定したファイルを解凍
	virtual status_t	MeltAll(const char *oDestFolder);							//書庫を全て解凍
	virtual status_t	SetTo(const entry_ref *oArcRef,uint32 oOpenMode);			//操作対象の書庫を指定
	virtual void		Unset();													//操作対象の書庫の指定を解除

	// 圧縮・解凍設定
	void	SetJunkPathsInFreezing(bool oJunk);
	bool	JunkPathsInFreezing() const;

	void	SetJunkPathsInMelting(bool oJunk);
	bool	JunkPathsInMelting() const;

	// ユーティリティー
	void	FormatTime(char *oDest,bigtime_t oTime,const char *oFormat) const;	//時刻を整形

	// トランスレータ部（オーバーライドしなくてもいいように、デフォルトの実装を提供）
	virtual status_t	Identify(BPositionIO *oInSource,const translation_format *oInFormat
			,BMessage *oIoExtension,translator_info *oOutInfo,uint32 oOutType);
	virtual const translation_format*	InputFormats(int32 *oOutCount) const;
	virtual const translation_format*	OutputFormats(int32 *oOutCount) const;
	virtual status_t	Translate(BPositionIO *oInSource,const translator_info *oInInfo,BMessage *oIoExtension
			,uint32 oOutType,BPositionIO *oOutDestination);
	virtual const char*	TranslatorInfo() const;
	virtual const char*	TranslatorName() const;
	virtual int32		TranslatorVersion() const;

protected:
	TTransArchiver();
	virtual	~TTransArchiver();

	// 圧縮・解凍設定
	bool	mJunkPathsInFreezing;	//圧縮時にパス情報を格納しない
	bool	mJunkPathsInMelting;	//解凍時にパス情報を無視

	// その他
	BFile	mArcFile;	//操作対象書庫

private:
	typedef	BTranslator	SUPER;

	void	FormatTimeReplace(BString *oString,const char *oOld,int oNew) const;	//FormatTime()の２桁数値置換用
};
//==============================================================================
extern "C" _EXPORT BTranslator*	make_nth_translator(int32 oN,image_id oYou,uint32 oFlags, ...);
//==============================================================================
#endif	//___TransArchiver_H_
