/*==============================================================================

	～～～　ＬＨＡ圧縮解凍心臓部用　メソッド別解凍処理　～～～

元のソース：

	LHa for UNIX Ver 1.14h + Ver 1.14i 差分

元のソースの著作権表示：

	LHAKernel.cpp を参照

ＢeＯＳ用に修正・新規作成：

	Modified		by ＳＨＩＮＴＡ (SHINTA)
	Ver 1.00		First Version 2000.12.17(Sun) by ＳＨＩＮＴＡ (SHINTA)
	Ver 1.01		少し整理した 2000.12.25(Mon) by ＳＨＩＮＴＡ (SHINTA)

==============================================================================*/

/*== MEMO ======================================================================
==============================================================================*/

#include "VPLHAKernel.h"
//------------------------------------------------------------------------------
#include <add/SHIDebug.h>
//------------------------------------------------------------------------------
//==============================================================================
TDecodeSet::~TDecodeSet()
{
}
//------------------------------------------------------------------------------
void	TDecodeSet::FillBuf(unsigned char n,BPositionIO *oInSrc,TMeltInfo *oMeltInfo)
{
	while ( n > oMeltInfo->mBitCount ) {
		n -= oMeltInfo->mBitCount;
		oMeltInfo->mBitBuf = (oMeltInfo->mBitBuf << oMeltInfo->mBitCount) + (oMeltInfo->mSubBitBuf >> (CHAR_BIT-oMeltInfo->mBitCount));
		if ( oMeltInfo->mLeftFrozenSize != 0) {
			oMeltInfo->mLeftFrozenSize--;
			oInSrc->Read(&oMeltInfo->mSubBitBuf,1);
		}
		else {
			oMeltInfo->mSubBitBuf = 0;
		}
		oMeltInfo->mBitCount = CHAR_BIT;
	}
	oMeltInfo->mBitCount -= n;
	oMeltInfo->mBitBuf = (oMeltInfo->mBitBuf << n) + (oMeltInfo->mSubBitBuf >> (CHAR_BIT-n));
	oMeltInfo->mSubBitBuf <<= n;
}
//------------------------------------------------------------------------------
unsigned short	TDecodeSet::GetBits(unsigned char n,BPositionIO *oInSrc,TMeltInfo *oMeltInfo)
{
	unsigned short	x;

	x = oMeltInfo->mBitBuf >> (2*CHAR_BIT-n);
	FillBuf(n,oInSrc,oMeltInfo);
	return x;
}
//------------------------------------------------------------------------------
void	TDecodeSet::InitGetBits(BPositionIO *oInSrc,TMeltInfo *oMeltInfo)
{
	oMeltInfo->mBitBuf = 0;
	oMeltInfo->mSubBitBuf = 0;
	oMeltInfo->mBitCount = 0;
	FillBuf(2*CHAR_BIT,oInSrc,oMeltInfo);
#ifdef EUC
	putc_euc_cache = EOF;
#endif
}
//------------------------------------------------------------------------------
status_t	TDecodeSet::MakeTable(short oNChar,unsigned char oBitLen[],short oTableBits,unsigned short oTable[],TMeltInfo *oMeltInfo)
{
	unsigned short	aCount[17];		// count of bitlen
	unsigned short	aWeight[17];	// 0x10000ul >> bitlen
	unsigned short	aStart[17];		// first code of bitlen
	unsigned short	aTotal;
	unsigned int	i,l;
	int				aAvail;
	int				j,k,m,n;
	unsigned short*	p;

	aAvail = oNChar;

	// initialize
	for ( i = 1 ; i <= 16 ; i++ ) {
		aCount[i] = 0;
		aWeight[i] = 1 << (16-i);
	}

	// count
	for ( i = 0 ; static_cast<short>(i) < oNChar ; i++ ) {
		aCount[oBitLen[i]]++;
	}

	// calculate first code
	aTotal = 0;
	for ( i = 1 ; i <= 16 ; i++ ) {
		aStart[i] = aTotal;
		aTotal += aWeight[i]*aCount[i];
	}
	if ( (aTotal & 0xFFFF) != 0 )
		return B_ERROR;	// error("make_table()", "Bad table (5)\n");

	// shift data for make table.
	m = 16-oTableBits;
	for ( i = 1 ; static_cast<short>(i) <= oTableBits; i++ ) {
		aStart[i] >>= m;
		aWeight[i] >>= m;
	}

	// initialize
	j = aStart[oTableBits+1] >> m;
	k = 1 << oTableBits;
	if ( j != 0 ) {
		for ( i = j ; static_cast<int>(i) < k ; i++ ) {
			oTable[i] = 0;
		}
	}

	// create table and tree
	for ( j = 0 ; j < oNChar ; j++ ) {
		k = oBitLen[j];
		if ( k == 0 )
			continue;
		l = aStart[k]+aWeight[k];
		if ( k <= oTableBits ) {
			// code in table
			for ( i = aStart[k] ; i < l ; i++ ) {
				oTable[i] = j;
			}
		} else {
			// code not in table
			p = &oTable[(i = aStart[k]) >> m];
			i <<= oTableBits;
			n = k-oTableBits;
			// make tree (n length)
			while ( --n >= 0 ) {
				if ( *p == 0 ) {
					oMeltInfo->mRight[aAvail] = oMeltInfo->mLeft[aAvail] = 0;
					*p = aAvail++;
				}
				if ( i & 0x8000 ) {
					p = &oMeltInfo->mRight[*p];
				} else {
					p = &oMeltInfo->mLeft[*p];
				}
				i <<= 1;
			}
			*p = j;
		}
		aStart[k] = l;
	}
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TDecodeSet::ReadCLen(BPositionIO *oInSrc,TMeltInfo *oMeltInfo)
{
	short	i, c, n;

	n = GetBits(CBIT,oInSrc,oMeltInfo);
	if ( n == 0 ) {
		c = GetBits(CBIT,oInSrc,oMeltInfo);
		for ( i = 0 ; i < NC ; i++ ) {
			oMeltInfo->mCLen[i] = 0;
		}
		for ( i = 0 ; i < 4096 ; i++ ) {
			oMeltInfo->mCTable[i] = c;
		}
	} else {
		i = 0;
		while ( i < n ) {
			c = oMeltInfo->mPtTable[oMeltInfo->mBitBuf >> (16-8)];
			if ( c >= NT ) {
				unsigned short	aMask = 1 << (16-9);
				do {
					if ( oMeltInfo->mBitBuf & aMask ) {
						c = oMeltInfo->mRight[c];
					 } else {
						c = oMeltInfo->mLeft[c];
					}
					aMask >>= 1;
				} while ( c >= NT );
			}
			FillBuf(oMeltInfo->mPtLen[c],oInSrc,oMeltInfo);
			if ( c <= 2 ) {
				if ( c == 0 ) {
					c = 1;
				} else if ( c == 1 ) {
					c = GetBits(4,oInSrc,oMeltInfo)+3;
				} else {
					c = GetBits(CBIT,oInSrc,oMeltInfo)+20;
				}
				while ( --c >= 0 ) {
					oMeltInfo->mCLen[i++] = 0;
				}
			} else {
				oMeltInfo->mCLen[i++] = c-2;
			}
		}
		while ( i < NC ) {
			oMeltInfo->mCLen[i++] = 0;
		}
		if ( MakeTable(NC,oMeltInfo->mCLen,12,oMeltInfo->mCTable,oMeltInfo) != B_OK )
			return B_ERROR;
	}
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TDecodeSet::ReadPtLen(short nn,short oNBit,short oISpecial,BPositionIO *oInSrc,TMeltInfo *oMeltInfo)
{
	int		i,c,n;

	n = GetBits(oNBit,oInSrc,oMeltInfo);
	if ( n == 0 ) {
		c = GetBits(oNBit,oInSrc,oMeltInfo);
		for ( i = 0 ; i < nn ; i++ ) {
			oMeltInfo->mPtLen[i] = 0;
		}
		for ( i = 0 ; i < 256 ; i++ ) {
			oMeltInfo->mPtTable[i] = c;
		}
	} else {
		i = 0;
		while ( i < n ) {
			c = oMeltInfo->mBitBuf >> (16-3);
			if ( c == 7 ) {
				unsigned short	aMask = 1 << (16-4);
				while ( aMask & oMeltInfo->mBitBuf ) {
					aMask >>= 1;
					c++;
				}
			}
			FillBuf((c < 7) ? 3 : c-3,oInSrc,oMeltInfo);
			oMeltInfo->mPtLen[i++] = c;
			if ( i == oISpecial ) {
				c = GetBits(2,oInSrc,oMeltInfo);
				while ( --c >= 0 ) {
					oMeltInfo->mPtLen[i++] = 0;
				}
			}
		}
		while ( i < nn ) {
			oMeltInfo->mPtLen[i++] = 0;
		}
		if ( MakeTable(nn,oMeltInfo->mPtLen,8,oMeltInfo->mPtTable,oMeltInfo) != B_OK )	// VU: 引数３つを oMeltInfo に統合できる？
			return B_ERROR;
	}
	return B_OK;
}
//==============================================================================
unsigned short	TLh5DecodeSet::DecodeC(BPositionIO *oInSrc,TMeltInfo *oMeltInfo)
/* ''''''''''''''''''''''''''''''
huf.c::decode_c_st1() 相当
   ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,, */
{
	unsigned short	aMask;
	unsigned short	j;

//	DBEXP("TLh5DecodeSet::DecodeC","");
	if (oMeltInfo->mBlockSize == 0) {
		oMeltInfo->mBlockSize = GetBits(16,oInSrc,oMeltInfo);
		ReadPtLen(NT,TBIT,3,oInSrc,oMeltInfo);
		ReadCLen(oInSrc,oMeltInfo);
		ReadPtLen(oMeltInfo->mNP,oMeltInfo->mPBit,-1,oInSrc,oMeltInfo);
	}
	oMeltInfo->mBlockSize--;
	j = oMeltInfo->mCTable[oMeltInfo->mBitBuf >> 4];
	if ( j < NC ) {
		FillBuf(oMeltInfo->mCLen[j],oInSrc,oMeltInfo);
	} else {
		FillBuf(12,oInSrc,oMeltInfo);
		aMask = 1 << (16-1);
		do {
			if ( oMeltInfo->mBitBuf & aMask ) {
				j = oMeltInfo->mRight[j];
			} else {
				j = oMeltInfo->mLeft[j];
			}
			aMask >>= 1;
		} while ( j >= NC );
		FillBuf(oMeltInfo->mCLen[j]-12,oInSrc,oMeltInfo);
	}
	return j;
}
//------------------------------------------------------------------------------
unsigned short	TLh5DecodeSet::DecodeP(BPositionIO *oInSrc,TMeltInfo *oMeltInfo)
/* ''''''''''''''''''''''''''''''
huf.c::decode_p_st1() 相当
   ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,, */
{
	unsigned short	aMask;
	unsigned short	j;

//	DBEXP("TLh5DecodeSet::DecodeP","");
	j = oMeltInfo->mPtTable[oMeltInfo->mBitBuf >> (16-8)];
	if ( j < oMeltInfo->mNP ) {
		FillBuf(oMeltInfo->mPtLen[j],oInSrc,oMeltInfo);
	} else {
		FillBuf(8,oInSrc,oMeltInfo);
		aMask = 1 << (16-1);
		do {
			if ( oMeltInfo->mBitBuf & aMask ) {
				j = oMeltInfo->mRight[j];
			} else {
				j = oMeltInfo->mLeft[j];
			}
			aMask >>= 1;
		} while ( j >= oMeltInfo->mNP );
		FillBuf(oMeltInfo->mPtLen[j]-8,oInSrc,oMeltInfo);
	}
	if ( j != 0 ) {
		j = (1 << (j-1)) + GetBits(j-1,oInSrc,oMeltInfo);
	}
	return j;
}
//------------------------------------------------------------------------------
void	TLh5DecodeSet::StartDecode(BPositionIO *oInSrc,TMeltInfo *oMeltInfo)
/* ''''''''''''''''''''''''''''''
huf.c::decode_start_st1() 相当
   ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,, */
{
//	DBEXP("TLh5DecodeSet::StartDecode","");
	if ( oMeltInfo->mDicBit <= 13 )  {
		oMeltInfo->mNP = 14;
		oMeltInfo->mPBit = 4;
	} else {
		if ( oMeltInfo->mDicBit == 16 ) {
			oMeltInfo->mNP = 17; // for -lh7-
		} else {
			oMeltInfo->mNP = 16;
		}
		oMeltInfo->mPBit = 5;
	}
	InitGetBits(oInSrc,oMeltInfo);
	oMeltInfo->mBlockSize = 0;
}
//==============================================================================
