/*==============================================================================
トランスＬＨＡアーカイバ　変換部
==============================================================================*/

/*== MEMO ======================================================================
==============================================================================*/

#include "TransLHAArchiver.h"
//------------------------------------------------------------------------------
// ＢeＯＳヘッダ
#include <Bitmap.h>
#include <Button.h>
#include <File.h>
#include <Mime.h>
#include <TranslationUtils.h>
#include <TranslatorFormats.h>
// Ｃヘッダ
#include <stdio.h>
#include <time.h>
// Ｃ++ヘッダ
// 追加ヘッダ
#include <add/SHIDebug.h>
// プロジェクトヘッダ
#include "LHAKernel.h"
#include "VPLHAKernel.h"
//==============================================================================
TTransLHAArchiver::TTransLHAArchiver()
{
	InstallMime();
}
//------------------------------------------------------------------------------
TTransLHAArchiver::~TTransLHAArchiver()
{
}
//------------------------------------------------------------------------------
bool	TTransLHAArchiver::FindFileName(const list<const char *> &oMeltFiles,const char *oTgtFName)
{
	list<const char *>::const_iterator	p = oMeltFiles.begin();

	while ( p != oMeltFiles.end() ) {
		if ( strcmp(*p,oTgtFName) == 0 )
			break;
		p++;
	}
	return (p != oMeltFiles.end());
}
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::GetAllIndividualInfos(list<TIndividualInfo> *oInfos)
{
	struct tm		aTM;
	TIndividualInfo	aInfo;
	TLzhHeader		aHdr;

	if ( mArcFile.InitCheck() != B_OK )
		return B_NO_INIT;
	// ＧＭＴとのずれを取得するために、ダミーの日付で mktime() を呼ぶ
	aTM.tm_year = 70;
	aTM.tm_mon = 0;
	aTM.tm_mday = 1;
	aTM.tm_hour = 0;
	aTM.tm_min = 0;
	aTM.tm_sec = 0;
	mktime(&aTM);
	// 先頭から順にヘッダを読む
	mArcFile.Seek(0,SEEK_SET);
	for ( ;; ) {
		// ヘッダの読み込み
		if ( aHdr.Scan(&mArcFile) != B_OK )
			break;
		// 情報の格納
		strcpy(aInfo.mFileName,aHdr.Name());
		strcpy(aInfo.mFormat,"LHA/");
		strncat(aInfo.mFormat,aHdr.MethodStr(),METHOD_TYPE_STRAGE);
		if ( aHdr.HasCrc() ) {
			aInfo.mCRC = aHdr.Crc();
		}
		aInfo.mOSType = aHdr.ExtendType();
		aInfo.mModificationTime = static_cast<bigtime_t>(aHdr.UnixLastModifiedStamp())*1000*1000;
		aInfo.mModificationTime += static_cast<bigtime_t>(aTM.tm_gmtoff)*1000*1000;
		aInfo.mFrozenSize = aHdr.FrozenSize();
		aInfo.mOriginalSize = aHdr.OriginalSize();
		oInfos->push_back(aInfo);
		// ファイルポインタを書庫内のファイルサイズ分進める
		mArcFile.Seek(aInfo.mFrozenSize,SEEK_CUR);
	}
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::Identify(BPositionIO *oInSource,const translation_format *oInFormat
		,BMessage *oIoExtension,translator_info *oOutInfo,uint32 oOutType)
		// oInSource を oOutType に変換できるか？　と聞かれるためのフック関数
		// できるなら、oOutInfo に詳細を格納し、B_OK を返す
		// できないのなら、B_NO_TRANSLATOR を返す
{
	status_t	aResult;

#ifdef DEBUG
	char	DB[16];
	sprintf(DB,"%c%c%c%c",(char)(oOutType >> 24),(char)(oOutType >> 16),(char)(oOutType >> 8),(char)oOutType);
#endif
	switch ( oOutType ) {
	case 0:	// デフォルト
	case B_TRANSLATOR_BITMAP:	// 中間画像へデコード
		aResult = IdentifyToBits(oInSource,oInFormat,oIoExtension,oOutInfo);
		DBEXP("TTransLHAArchiver::Identify - 扱える形式 a",DB);
		break;
	case TRANS_LHA_ARCHIVER_TGT_TYPE:	// 圧縮
		aResult = IdentifyToLzh(oInSource,oInFormat,oIoExtension,oOutInfo);
		DBEXP("TTransLHAArchiver::Identify - 扱える形式 b",DB);
		break;
	case TRANS_ARCHIVER_RAW_TYPE:	// 解凍
		aResult = IdentifyToRaw(oInSource,oInFormat,oIoExtension,oOutInfo);
		DBEXP("TTransLHAArchiver::Identify - 扱える形式 c",DB);
		break;
	default:	//扱えない
		DBEXP("TTransLHAArchiver::Identify - 扱えない形式",DB);
		aResult = B_NO_TRANSLATOR;
	}
	return aResult;
}
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::IdentifyToBits(BPositionIO *oInSource,const translation_format *oInFormat
		,BMessage *oIoExtension,translator_info *oOutInfo)
		// Lzh -> Bits が可能かどうか判定
{
	TLzhHeader	aHdr;

	oInSource->Seek(0,SEEK_SET);
	if ( aHdr.Scan(oInSource) != B_OK ) {
		DBEXP("TTransLHAArchiver::IdentifyZipToBits","デコード不可");
		return B_NO_TRANSLATOR;
	}
	DBEXP("TTransLHAArchiver::IdentifyZipToBits","デコード可");
	oOutInfo->group = B_TRANSLATOR_BITMAP;	// type よりも大きな分類
	oOutInfo->type = B_TRANSLATOR_BITMAP;
	oOutInfo->quality = TRANS_LHA_ARCHIVER_QUALITY;	//??:元のデータをどの程度エンコードできるか(?)
	oOutInfo->capability = TRANS_LHA_ARCHIVER_CAPACITY;	//??:どの程度デコードできるか(?)
	strcpy(oOutInfo->name,BITMAP_NAME);
	strcpy(oOutInfo->MIME,BITMAP_MIME);
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::IdentifyToLzh(BPositionIO *oInSource,const translation_format *oInFormat
		,BMessage *oIoExtension,translator_info *oOutInfo)
		// Bits -> Lzh が可能かどうか判定
{
	oOutInfo->group = TRANS_ARCHIVER_RAW_TYPE;	// type よりも大きな分類
	oOutInfo->type = TRANS_LHA_ARCHIVER_TGT_TYPE;
	oOutInfo->quality = TRANS_LHA_ARCHIVER_QUALITY;	//??:元のデータをどの程度エンコードできるか(?)
	oOutInfo->capability = TRANS_LHA_ARCHIVER_CAPACITY;	//??:どの程度デコードできるか(?)
	strcpy(oOutInfo->name,TRANS_LHA_ARCHIVER_TGT_NAME);
	strcpy(oOutInfo->MIME,TRANS_LHA_ARCHIVER_TGT_MIME);
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::IdentifyToRaw(BPositionIO *oInSource,const translation_format *oInFormat
		,BMessage *oIoExtension,translator_info *oOutInfo)
		// Lzh -> Raw（解凍） が可能かどうか判定
{
	return IdentifyToBits(oInSource,oInFormat,oIoExtension,oOutInfo);
}
//------------------------------------------------------------------------------
const translation_format*	TTransLHAArchiver::InputFormats(int32 *oOutCount) const
		//読み込める形式
{
	static translation_format	saInputFmts[] = {
			{ TRANS_LHA_ARCHIVER_TGT_TYPE,TRANS_ARCHIVER_RAW_TYPE,TRANS_LHA_ARCHIVER_QUALITY,TRANS_LHA_ARCHIVER_CAPACITY,TRANS_LHA_ARCHIVER_TGT_MIME,TRANS_LHA_ARCHIVER_TGT_NAME },	//ネイティブ
//			{ B_TRANSLATOR_BITMAP,B_TRANSLATOR_BITMAP,TRANS_LHA_ARCHIVER_QUALITY,TRANS_LHA_ARCHIVER_CAPACITY,BITMAP_MIME,BITMAP_NAME },	//画像も扱える
	};
	DBEXP("TTransLHAArchiver::InputFormats","");
	*oOutCount = 1;
	return saInputFmts;
}
//------------------------------------------------------------------------------
void	TTransLHAArchiver::InstallMime()
{
//	BBitmap*	aIcon;
	BMessage	aExts;
	BMimeType	aMime(TRANS_LHA_ARCHIVER_TGT_MIME);

	if ( aMime.IsInstalled() )
		return;
	// まだ登録されていないので、登録する
		// 拡張子
	aExts.AddString(MSGNAME_EXTS,LHA_ARCHIVE_EXT);
	aExts.AddString(MSGNAME_EXTS,BString(LHA_ARCHIVE_EXT).ToUpper());
	aMime.SetFileExtensions(&aExts);
		// 説明
	aMime.SetShortDescription("LHA Archive");
	aMime.SetLongDescription("LHA Archive (.lzh file)");
#if 0
		// アイコン（アプリケーション側ではなく、データ側にアイコンを関連づけてしまう）
	aIcon = BTranslationUtils::GetBitmap(B_RAW_TYPE,1000);
	DBEXP("aIcon - TTransLHAArchiver::InstallMime",(const void *)aIcon);
	DBEXPIF(aIcon->ColorSpace() == B_CMAP8,"TTransLHAArchiver::InstallMime","Icon is CMAP8");
	DBEXP("IconSize - TTransLHAArchiver::InstallMime",aIcon->Bounds());
	aMime.SetIcon(aIcon,B_LARGE_ICON);
	delete aIcon;
#endif
		// 登録
	DBEXP("TTransLHAArchiver::InstallMime","LHA書庫タイプ登録");
	aMime.Install();
}
//------------------------------------------------------------------------------
#if 0
status_t	TTransLHAArchiver::MakeConfigurationView(BMessage *oIoExtension,BView **oOutView
		,BRect *oOutExtent)
{
	BButton*	aButton = new BButton(BRect(20,20,70,40),"Button","Button",NULL);
	BView*		aView = new BView(BRect(0,0,300,100),"MyTranslator Settings",B_FOLLOW_ALL,B_WILL_DRAW);
	
	aView->AddChild(aButton);
	*oOutView = aView;
	*oOutExtent = aView->Bounds();
	return B_OK;
}
#endif
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::Melt(const list<const char *> &oMeltFiles,const char *oDestFolder)
{
	status_t		aResult = B_OK;
	TLHAKernel		aLHAKernel;
	TLzhHeader		aHdr;

	if ( mArcFile.InitCheck() != B_OK )
		return B_NO_INIT;
	// 先頭から順にヘッダを読む
	aLHAKernel.SetTo(&mArcFile);
	for ( ;; ) {
		// ヘッダの読み込み
		if ( aLHAKernel.GetHeader(&aHdr) == TLA_ERR_EOF )
			break;
		if ( FindFileName(oMeltFiles,aHdr.Name()) ) {	// 解凍リストに載っている
			aResult |= aLHAKernel.Extract(oDestFolder);
		} else {	// ファイルポインタを書庫内のファイルサイズ分進める
			aLHAKernel.SeekNext();
		}
	}
	return aResult;
}
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::MeltAll(const char *oDestFolder)
{
	status_t		aResult = B_OK;
	TLHAKernel		aLHAKernel;
	TLzhHeader		aHdr;

	if ( mArcFile.InitCheck() != B_OK )
		return B_NO_INIT;
	// 先頭から順にヘッダを読む
	aLHAKernel.SetTo(&mArcFile);
	for ( ;; ) {
		// ヘッダの読み込み
		if ( aLHAKernel.GetHeader(&aHdr) == TLA_ERR_EOF )
			break;
		// 解凍
		aResult |= aLHAKernel.Extract(oDestFolder);
	}
	return aResult;
}
//------------------------------------------------------------------------------
const translation_format*	TTransLHAArchiver::OutputFormats(int32 *oOutCount) const
		//書き出せる形式
{
	static translation_format	saOutputFmts[] = {
//			{ TRANS_LHA_ARCHIVER_TGT_TYPE,TRANS_ARCHIVER_RAW_TYPE,TRANS_LHA_ARCHIVER_QUALITY,TRANS_LHA_ARCHIVER_CAPACITY,TRANS_LHA_ARCHIVER_TGT_MIME,TRANS_LHA_ARCHIVER_TGT_NAME },	//ネイティブ
//			{ B_TRANSLATOR_BITMAP,B_TRANSLATOR_BITMAP,TRANS_LHA_ARCHIVER_QUALITY,TRANS_LHA_ARCHIVER_CAPACITY,BITMAP_MIME,BITMAP_NAME },	//画像も扱える
	};
	DBEXP("TTransLHAArchiver::OutputFormats","");
	*oOutCount = 0;
	return saOutputFmts;
}
//------------------------------------------------------------------------------
int32	TTransLHAArchiver::SubClassCompatiIdent() const
{
	return COMPATIBILITY_IDENT;
}
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::Translate(BPositionIO *oInSource,const translator_info *oInInfo,BMessage *oIoExtension
		,uint32 oOutType,BPositionIO *oOutDestination)
{
	status_t	aResult;
	
//	DBEXP("TTransLHAArchiver::Translate",system("BeCopy /boot/home/WorkData/MyTga /boot/home"));
	switch ( oOutType ) {
	case TRANS_LHA_ARCHIVER_TGT_TYPE:	//エンコード
		aResult = TranslateBitsToLzh(oInSource,oInInfo,oIoExtension,oOutDestination);
		break;
	case 0:	//デフォルト
	case B_TRANSLATOR_BITMAP:	//デコード
		aResult = TranslateLzhToBits(oInSource,oInInfo,oIoExtension,oOutDestination);
		break;
	default:
		aResult = B_NO_TRANSLATOR;
	}
	return aResult;
}
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::TranslateBitsToLzh(BPositionIO *oInSource,const translator_info *oInInfo,BMessage *oIoExtension
		,BPositionIO *oOutDestination)
{
#if 0
	char				aCmdLine[B_FILE_NAME_LENGTH*2];
	char				aVarDirName[B_FILE_NAME_LENGTH];	// ランダム（フルパス）
	char				aVarFName[B_FILE_NAME_LENGTH];		// aVarDirName+STORE_FNAME
	char				aVarZipFName[B_FILE_NAME_LENGTH];	// aVarFName+".zip"
	uint8*				aBuf;
	uint8*				aBitsHeaderPtr;
	off_t				aInSize;
	BFile				aVarFile;
	TranslatorBitmap	aBitsHeader;

	DBEXP("TTransZipArchiver::TranslateBitsToLzh","--->");
	//入力サイズ
	oInSource->Seek(0,SEEK_END);
	aInSize = oInSource->Position();
	aBuf = new uint8 [aInSize];
	//ヘッダを読み込む（aBitsHeader.magicはエンディアン変換しない）
	aBitsHeaderPtr = reinterpret_cast<uint8 *>(&aBitsHeader);
	oInSource->Seek(0,SEEK_SET);
	oInSource->Read(aBitsHeaderPtr,sizeof(aBitsHeader));
	aBitsHeader.bounds.left = B_BENDIAN_TO_HOST_FLOAT(aBitsHeader.bounds.left);
	aBitsHeader.bounds.right = B_BENDIAN_TO_HOST_FLOAT(aBitsHeader.bounds.right);
	aBitsHeader.bounds.top = B_BENDIAN_TO_HOST_FLOAT(aBitsHeader.bounds.top);
	aBitsHeader.bounds.bottom = B_BENDIAN_TO_HOST_FLOAT(aBitsHeader.bounds.bottom);
	aBitsHeader.rowBytes = B_BENDIAN_TO_HOST_INT32(aBitsHeader.rowBytes);
	aBitsHeader.colors = (color_space)B_BENDIAN_TO_HOST_INT32(aBitsHeader.colors);
	aBitsHeader.dataSize = B_BENDIAN_TO_HOST_INT32(aBitsHeader.dataSize);
	//一時ファイルにビッツを書き出す
	aVarFile.SetTo(aVarFName,B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
	aVarFile.Write(aBitsHeaderPtr,sizeof(aBitsHeader));	//ヘッダの書き出し
	aInSize = oInSource->Read(aBuf,aInSize);
	aVarFile.Write(aBuf,aInSize);	//内容の書きだし
	aVarFile.Unset();
	delete aBuf;
	//一時ファイルを圧縮
	sprintf(aCmdLine,"zip -j %s %s",aVarZipFName,aVarFName);
	if ( system(aCmdLine) != 0 )
		return B_NO_TRANSLATOR;
	//圧縮ファイルサイズ
	aVarFile.SetTo(aVarZipFName,B_READ_ONLY);
	aVarFile.GetSize(&aInSize);
	aBuf = new uint8 [aInSize];
	//一時ファイルからアーカイブの内容を読み込んで目的バッファにかきだし
	aVarFile.Read(aBuf,aInSize);
	oOutDestination->Seek(0,SEEK_SET);
	oOutDestination->Write(aBuf,aInSize);
	delete aBuf;
	//一時ファイルを削除
	sprintf(aCmdLine,"rm %s/*",aVarDirName);
	system(aCmdLine);
	sprintf(aCmdLine,"rmdir %s",aVarDirName);
	system(aCmdLine);
	return B_OK;
#endif
	return B_NO_TRANSLATOR;
}
//------------------------------------------------------------------------------
status_t	TTransLHAArchiver::TranslateLzhToBits(BPositionIO *oInSource,const translator_info *oInInfo,BMessage *oIoExtension
		,BPositionIO *oOutDestination)
{
	return B_NO_TRANSLATOR;
}
//------------------------------------------------------------------------------
const char*	TTransLHAArchiver::TranslatorInfo() const
{
	static char	saInfo[] = "Trans LHA Archiver by SHINTA\nOriginal LHA kernel : LHa for UNIX by t.okamoto etc.";

	return saInfo;
}
//------------------------------------------------------------------------------
const char*	TTransLHAArchiver::TranslatorName() const
{
	static char	saName[] = "Trans LHA Archiver";

	return saName;
}
//------------------------------------------------------------------------------
int32		TTransLHAArchiver::TranslatorVersion() const
{
	return 100;
}
//==============================================================================
BTranslator*	make_nth_translator(int32 oN,image_id oYou,uint32 oFlags, ...)
{
	if ( oN == 0 )
		return new TTransLHAArchiver();
	else
		return NULL;
}
//==============================================================================
