#ifndef	___VarPackLHAKernel_H_
#define	___VarPackLHAKernel_H_
//------------------------------------------------------------------------------
// ＢeＯＳヘッダ
#include <DataIO.h>
#include <StorageDefs.h>
// Ｃ++ヘッダ
#include <string>
// プロジェクトヘッダ
#include "DecodeSet.h"
#include "LHAConsts.h"
#include "TransArchiver.h"
//------------------------------------------------------------------------------
// エラーコード
enum {
	TLA_ERR_EOF = TRANS_ARCHIVER_ERR_BASE
};
//------------------------------------------------------------------------------
using namespace	std;
//==============================================================================
class	TLzhHeader
		// ヘッダに格納されている情報を保持する
{
public:
	TLzhHeader();
	~TLzhHeader();

	status_t		Scan(BPositionIO *oInSrc);		// ヘッダを読んで情報のセット

	// 情報取得
	inline unsigned short	Crc() const { return mCrc; };
	inline unsigned char	ExtendType() const { return mExtendType; };
	inline bool				HasCrc() const { return mHasCrc; };
	inline int				MethodNum() const { return mMethodNum; };
	inline const char*		MethodStr() const { return mMethodStr; };
	inline const char*		Name() const { return mName; };
	inline long				OriginalSize() const { return mOriginalSize; };
	inline long				FrozenSize() const { return mFrozenSize; };
	inline time_t			UnixLastModifiedStamp() const { return mUnixLastModifiedStamp; };
	inline unsigned short	UnixMode() const { return mUnixMode; };

private:
	void			CalcMethodNum();				// メソッド文字列からメソッド番号を算出
	int				CalcSum(char *oPtr,int oLen);
	char*			ConvDelim(char *oPath,char oDelim);
	void			GenericToUnixFileName(char *oName,int oLen);
	time_t			GenericToUnixStamp(long oGenericTime);

	// lha_macro.h より
	int				GetByte();
	long			GetLongword();
	char*			GetPtr();
	unsigned short	GetWord();
	void			SetupGet(char *oPtr);

	bool			mHasCrc;
	char			mMethodStr[METHOD_TYPE_STRAGE];	// メソッドを表す文字列（"-lh5-"など）（NULL は付かない）
	char			mName[B_FILE_NAME_LENGTH];		// ファイル名（パス付き）
	char*			mHdrPtr;						// SetupGet() などで使用
	unsigned char	mDosFileAttr;
	unsigned char	mExtendType;
	unsigned char	mHdrLevel;						// ヘッダレベル（0,1,2）
	unsigned char	mMinorVersion;
	unsigned short	mCrc;							// ヘッダに書き込まれているＣＲＣ
	unsigned short	mUnixGid;
	unsigned short	mUnixMode;						// UNIX パーミッション
	unsigned short	mUnixUid;
	int				mMethodNum;						// メソッドを表す番号
	long			mDosLastModifiedStamp;			// 更新日時
	long			mOriginalSize;					// 元のファイルサイズ
	long			mFrozenSize;					// 圧縮後のファイルサイズ
	time_t			mUnixLastModifiedStamp;			// 更新日時
};
//==============================================================================
class	TMeltInfo
		// 解凍時の情報
{
public:
	TMeltInfo(const TLzhHeader *oHdr);
	~TMeltInfo();

	unsigned short	DecodeC(BPositionIO *oInSrc);
	unsigned short	DecodeP(BPositionIO *oInSrc);
	void			SetDecodeSet(TDecodeSet *oDecodeSet);
	void			StartDecode(BPositionIO *oInSrc);

	bool				mVerifyMode;
	unsigned char		mBitCount;
	unsigned char		mSubBitBuf;
	unsigned short		mBitBuf;
	unsigned short		mBlockSize;
	unsigned short		mRealCrc;	// 実際に計算したＣＲＣ
	int					mDicBit;	// DecodeLzhuf() で設定し、以降不変（多分）
	int					mNP;
	int					mPBit;
	int					mPrevChar;
	unsigned long		mCount;
	unsigned long		mLeftFrozenSize;
	unsigned long		mLoc;
	long				mReadingSize;
	const TLzhHeader*	mHdr;		// ヘッダの内容へのポインタ

	// 各種テーブル
	unsigned char		mCLen[NC];
	unsigned char		mPtLen[NPT];
	unsigned short		mCrcTable[UCHAR_MAX+1];
	unsigned short		mCTable[4096];
	unsigned short		mLeft[2*NC-1],mRight[2*NC-1];
	unsigned short		mPtTable[256];
private:
	void				MakeCrcTable();

	TDecodeSet*			mDecodeSet;	// 各形式での処理分岐用（TMeltInfo が所有する）
};
//==============================================================================
class	TException	//例外
{
private:
	int32		mi3ID;				//例外ＩＤ
	string		mxString;			//内容説明などの文字列
public:
	TException(int32 i3ID,const char *cString = "");
	TException(const char *cString);
	~TException();
	
	const char*	String() const;		//説明を得る
	int32		ID() const;			//ＩＤを返す
};
//==============================================================================
#endif	//___VarPackLHAKernel_H_
