/*==============================================================================
トランスＺipアーカイバ　変換部
==============================================================================*/

/*== MEMO ======================================================================
==============================================================================*/

#include "TransZipArchiver.h"
//------------------------------------------------------------------------------
// ＢeＯＳヘッダ
#include <Button.h>
#include <Directory.h>
#include <Entry.h>
#include <File.h>
#include <FindDirectory.h>
#include <Locker.h>
#include <Path.h>
#include <TranslatorFormats.h>
#include <View.h>
// Ｃヘッダ
#include <stdio.h>
#include <stdlib.h>
// Ｃ++ヘッダ
#include <fstream>
#include <strstream>
// 追加ヘッダ
#include <add/SHIDebug.h>
#include <add/SHIString.h>
//------------------------------------------------------------------------------
using namespace	std;
//==============================================================================
TTransZipArchiver::TTransZipArchiver()
{
	Unset();
}
//------------------------------------------------------------------------------
TTransZipArchiver::~TTransZipArchiver()
{
}
//------------------------------------------------------------------------------
void	TTransZipArchiver::DeleteFolder(const char *oFolderName)
{
	BEntry		aDelEntry;
	BDirectory	aFolder(oFolderName);

	while ( aFolder.GetNextEntry(&aDelEntry) == B_OK ) {
		aDelEntry.Remove();
	}
	BEntry(oFolderName).Remove();
}
//------------------------------------------------------------------------------
string	TTransZipArchiver::EliminateBaseFolder(const char *oTgtFName,const char *oBaseFolder)
/*	''''''''''''''''''''''''''''''
	パスが同じ部分を削る
	直後の'/'も削る
,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,	*/
{
	int i = 0;

	if ( oBaseFolder == NULL )	//基準フォルダが指定されていない場合
		return string(oTgtFName);
//	if ( oTgtFName[0] != oBaseFolder[0] )	//最初から違う場合
//		return string(oTgtFName);
	while ( oTgtFName[i] != '\0' && oTgtFName[i] == oBaseFolder[i] ) {
		i++;
	}
	if ( oTgtFName[i] == '/' ) {
		i++;	//相対パスにするために'/'を削る
	}
	return string(oTgtFName+i);
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::Freeze(const list<const char *> &oFreezeFiles,const char *oBaseFolder)
{
	bool		aZipExtNo = false;
	char		aTmpFolderName[B_FILE_NAME_LENGTH];
	list<const char*>::const_iterator	p = oFreezeFiles.begin();
	off_t		aArcSize;
	string		aCmdLine;
	string		aArcFStr = mArcFName;
	BDirectory	aOrgFolder;
	BDirectory	aTmpFolder;
	BEntry		aArcEntry(mArcFName);
	BPath		aOrgFolderPath;

	if ( !IsSet() )
		return B_ERROR;
	// 拡張子が無い場合は、テンポラリファイルに圧縮
	if ( aArcFStr.rfind(FPERIEXT_ZIP) != aArcFStr.length()-strlen(FPERIEXT_ZIP) ) {
		GenerateTmpFName(aTmpFolderName);
		create_directory(aTmpFolderName,0777);
		aTmpFolder.SetTo(aTmpFolderName);
		aArcEntry.MoveTo(&aTmpFolder,(aArcFStr+FPERIEXT_ZIP).c_str(),true);
		aZipExtNo = true;
	}
	//ファイルが０バイトだったら削除（０バイトのファイルに追加できないため）
	if ( aArcEntry.GetSize(&aArcSize) == B_OK && aArcSize == 0 ) {
		aArcEntry.Remove();
	}
	//圧縮（ファイル１つずつ）
	while ( p != oFreezeFiles.end() ) {
		if ( oBaseFolder == NULL ) {	//基準フォルダが無いので、フルパスで格納
			aCmdLine = "";
		} else {	//基準フォルダからの相対パスで格納
			aCmdLine = string("cd \"")+oBaseFolder+"\"\n";	//基準フォルダに移動
		}
		aCmdLine += string("zip -9 -y \"")+BPath(&aArcEntry).Path()+"\" \""+EliminateBaseFolder(*p,oBaseFolder)+"\"";
		DBEXP("aCmdLine - TTransZipArchiver::Freeze",aCmdLine);
		system(aCmdLine.c_str());
		p++;
	}
	// 拡張子を戻す
	if ( aZipExtNo ) {
		BPath(mArcFName).GetParent(&aOrgFolderPath);
		aOrgFolder.SetTo(aOrgFolderPath.Path());
		aArcEntry.MoveTo(&aOrgFolder,BPath(mArcFName).Leaf(),true);
		DeleteFolder(aTmpFolderName);
	}
	return B_OK;
}
//------------------------------------------------------------------------------
void		TTransZipArchiver::GenerateTmpFName(char *oDest) const
{
	char	aVarFNameSuf[VAR_FNAME_SUF_LENGTH+1];
	BPath	aVarDirPath;

	find_directory(B_COMMON_TEMP_DIRECTORY,&aVarDirPath,true);
	srand(system_time()/(1000*1000) << 16 | system_time() & 0xFFFF);	// 秒オーダーとμ秒オーダーの両方を使用して衝突を防ぐ
	for ( int i = 0 ; i < VAR_FNAME_SUF_LENGTH ; i++ ) {
		aVarFNameSuf[i] = rand()%26+'a';
	}
	aVarFNameSuf[VAR_FNAME_SUF_LENGTH] = '\0';
	sprintf(oDest,"%s/%s%s",aVarDirPath.Path(),VAR_FNAME_BASE,aVarFNameSuf);
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::GetAllIndividualInfos(list<TIndividualInfo> *oInfos)
{
	char			aCmdLine[B_FILE_NAME_LENGTH*2];
	char			aDate[32];
	char			aLineBuf[B_FILE_NAME_LENGTH*2];
	char			aTime[32];
	char			aTmpDirName[B_FILE_NAME_LENGTH];
	char			aTmpListFName[B_FILE_NAME_LENGTH];
	ifstream		aIn;
	istrstream*		aStrStream;
	struct tm		aTM;
	TIndividualInfo	aInfo;

	if ( !IsSet() )
		return B_NO_INIT;
	//ファイル・ディレクトリの生成
	GenerateTmpFName(aTmpDirName);
	sprintf(aTmpListFName,"%s/%s",aTmpDirName,LIST_FNAME);
	sprintf(aCmdLine,"mkdir %s",aTmpDirName);
	system(aCmdLine);
	//リストファイルの生成
	sprintf(aCmdLine,"unzip -l \"%s\" > %s",mArcFName,aTmpListFName);
	system(aCmdLine);
	//リストの作成
	aIn.open(aTmpListFName,ios::in);
		//ヘッダの読み飛ばし
	for ( int i = 0 ; i < 3 ; i++ ) {
		aIn.getline(aLineBuf,sizeof(aLineBuf));
	}
		//リストの作成
	for ( ;; ) {
		aIn.getline(aLineBuf,sizeof(aLineBuf));
		//VU:EOFチェック
		if ( aIn.eof() || strcmp(aLineBuf," --------                   -------") == 0 )
			break;
		aStrStream = new istrstream(aLineBuf);
		*aStrStream >> aInfo.mOriginalSize >> aDate >> aTime;
//		DBEXP("TTransZipArchiver::GetAllIndividualInfos - pos",aStrStream->peek());
		strcpy(aInfo.mFileName,&aLineBuf[28]);
		strcpy(aInfo.mFormat,"Zip/-----");
		//更新日時の取得
		aTM.tm_year = atoi(&aDate[6]);	//tm_yearは1900年からの経過、aDateは西暦下2桁
		if ( aTM.tm_year < TRANS_ARCHIVER_YEAR_BORDER ) {	//2000年代
			aTM.tm_year += 100;
		}
//		DBEXP("year - TTransZipArchiver::GetAllIndividualInfos",aTM.tm_year);
		aTM.tm_mon = atoi(&aDate[0])-1;	//tm_monは0ベース、aDateは1ベース
//		DBEXP("mon - TTransZipArchiver::GetAllIndividualInfos",aTM.tm_mon);
		aTM.tm_mday = atoi(&aDate[3]);	//tm_mdayは1ベース、aDateは1ベース
//		DBEXP("mday - TTransZipArchiver::GetAllIndividualInfos",aTM.tm_mday);
		aTM.tm_hour = atoi(&aTime[0]);	//tm_hourは0ベース、aTimeは0ベース
//		DBEXP("hour - TTransZipArchiver::GetAllIndividualInfos",aTM.tm_hour);
		aTM.tm_min = atoi(&aTime[3]);	//tm_minは0ベース、aTimeは0ベース
//		DBEXP("min - TTransZipArchiver::GetAllIndividualInfos",aTM.tm_min);
		aTM.tm_sec = 0;
		aTM.tm_isdst = 0;	//??:よくわからない
		aInfo.mModificationTime = mktime(&aTM);//*1000;
//		DBEXP("time - TTransZipArchiver::GetAllIndividualInfos",aInfo.mModifyTime);
		aInfo.mModificationTime *= 1000*1000;
//		DBEXP("time - TTransZipArchiver::GetAllIndividualInfos",aInfo.mModifyTime);
		aInfo.mModificationTime += bigtime_t(aTM.tm_gmtoff)*1000*1000;	//Local->GMT
//		DBEXP("time - TTransZipArchiver::GetAllIndividualInfos",aInfo.mModifyTime);
		//情報をリストに格納
		oInfos->push_back(aInfo);
		delete aStrStream;
	}
	DeleteFolder(aTmpDirName);
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::Identify(BPositionIO *oInSource,const translation_format *oInFormat
		,BMessage *oIoExtension,translator_info *oOutInfo,uint32 oOutType)
		// oInSource を oOutType に変換できるか？　と聞かれるためのフック関数
		// できるなら、oOutInfo に詳細を格納し、B_OK を返す
		// できないのなら、B_NO_TRANSLATOR を返す
{
	status_t	aResult;

#ifdef DEBUG
	char	DB[16];
	sprintf(DB,"%c%c%c%c",(char)(oOutType >> 24),(char)(oOutType >> 16),(char)(oOutType >> 8),(char)oOutType);
#endif
	switch ( oOutType ) {
	case 0:	//デフォルト
	case B_TRANSLATOR_BITMAP:	//中間画像へデコード
		aResult = IdentifyToBits(oInSource,oInFormat,oIoExtension,oOutInfo);
		DBEXP("TTransZipArchiver::Identify - 扱える形式",DB);
		break;
	case TRANS_ZIP_ARCHIVER_TGT_TYPE:	//圧縮
		aResult = IdentifyToZip(oInSource,oInFormat,oIoExtension,oOutInfo);
		DBEXP("TTransZipArchiver::Identify - 扱える形式",DB);
		break;
	case TRANS_ARCHIVER_RAW_TYPE:	//解凍
		aResult = IdentifyToRaw(oInSource,oInFormat,oIoExtension,oOutInfo);
		DBEXP("TTransZipArchiver::Identify - 扱える形式",DB);
		break;
	default:	//扱えない
		DBEXP("TTransZipArchiver::Identify - 扱えない形式",DB);
		aResult = B_NO_TRANSLATOR;
	}
	return aResult;
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::IdentifyToBits(BPositionIO *oInSource,const translation_format *oInFormat
		,BMessage *oIoExtension,translator_info *oOutInfo)
		// Zip -> Bits が可能かどうか判定
{
	char	aBuf[32];
	char	aZipHeader[] = { 0x50,0x4B,0x03,0x04,0x00 };	//ＺＩＰ書庫のヘッダ
	
	oInSource->Seek(0,SEEK_SET);
	oInSource->Read(aBuf,sizeof(aBuf));
	if ( strncmp(aBuf,aZipHeader,strlen(aZipHeader)) != 0 ) {	//ヘッダがＺＩＰ書庫のものではない
		DBEXP("TTransZipArchiver::IdentifyZipToBits","デコード不可");
		return B_NO_TRANSLATOR;
	}
	DBEXP("TTransZipArchiver::IdentifyZipToBits","デコード可");
	oOutInfo->group = B_TRANSLATOR_BITMAP;	// type よりも大きな分類
	oOutInfo->type = B_TRANSLATOR_BITMAP;
	oOutInfo->quality = TRANS_ZIP_ARCHIVER_QUALITY;	//??:元のデータをどの程度エンコードできるか(?)
	oOutInfo->capability = TRANS_ZIP_ARCHIVER_CAPACITY;	//??:どの程度デコードできるか(?)
	strcpy(oOutInfo->name,BITMAP_NAME);
	strcpy(oOutInfo->MIME,BITMAP_MIME);
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::IdentifyToZip(BPositionIO *oInSource,const translation_format *oInFormat
		,BMessage *oIoExtension,translator_info *oOutInfo)
		// Bits -> Zip が可能かどうか判定
{
	oOutInfo->group = TRANS_ARCHIVER_RAW_TYPE;	// type よりも大きな分類
	oOutInfo->type = TRANS_ZIP_ARCHIVER_TGT_TYPE;
	oOutInfo->quality = TRANS_ZIP_ARCHIVER_QUALITY;	//??:元のデータをどの程度エンコードできるか(?)
	oOutInfo->capability = TRANS_ZIP_ARCHIVER_CAPACITY;	//??:どの程度デコードできるか(?)
	strcpy(oOutInfo->name,TRANS_ZIP_ARCHIVER_TGT_NAME);
	strcpy(oOutInfo->MIME,TRANS_ZIP_ARCHIVER_TGT_MIME);
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::IdentifyToRaw(BPositionIO *oInSource,const translation_format *oInFormat
		,BMessage *oIoExtension,translator_info *oOutInfo)
		// Zip -> Raw（解凍） が可能かどうか判定
{
	return IdentifyToBits(oInSource,oInFormat,oIoExtension,oOutInfo);
}
//------------------------------------------------------------------------------
const translation_format*	TTransZipArchiver::InputFormats(int32 *oOutCount) const
		//読み込める形式
{
	static translation_format	saInputFmts[] = {
			{ TRANS_ZIP_ARCHIVER_TGT_TYPE/*type*/,TRANS_ARCHIVER_RAW_TYPE/*gruop*/,TRANS_ZIP_ARCHIVER_QUALITY,TRANS_ZIP_ARCHIVER_CAPACITY,TRANS_ZIP_ARCHIVER_TGT_MIME,TRANS_ZIP_ARCHIVER_TGT_NAME },	//ネイティブ
			{ B_TRANSLATOR_BITMAP,B_TRANSLATOR_BITMAP,TRANS_ZIP_ARCHIVER_QUALITY,TRANS_ZIP_ARCHIVER_CAPACITY,BITMAP_MIME,BITMAP_NAME },	//画像も扱える
	};
	DBEXP("TTransZipArchiver::InputFormats","");
	*oOutCount = 2;
	return saInputFmts;
}
//------------------------------------------------------------------------------
bool	TTransZipArchiver::IsSet() const
{
	return (mArcFName[0] != '\0');
}
//------------------------------------------------------------------------------
void	TTransZipArchiver::MakeCommonMeltCmd(string *oCmdLine)
{
	*oCmdLine = "unzip -o ";
	if ( mJunkPathsInMelting ) {	//パスを無視
		(*oCmdLine) += "-j ";
	}
}
//------------------------------------------------------------------------------
#if 0
status_t	TTransZipArchiver::MakeConfigurationView(BMessage *oIoExtension,BView **oOutView
		,BRect *oOutExtent)
{
	BButton*	aButton = new BButton(BRect(20,20,70,40),"Button","Button",NULL);
	BView*		aView = new BView(BRect(0,0,300,100),"MyTranslator Settings",B_FOLLOW_ALL,B_WILL_DRAW);
	
	aView->AddChild(aButton);
	*oOutView = aView;
	*oOutExtent = aView->Bounds();
	return B_OK;
}
#endif
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::Melt(const list<const char *> &oMeltFiles,const char *oDestFolder)
{
	list<const char *>::const_iterator	p;
	string		aCmdLine;

	if ( !IsSet() )
		return B_NO_INIT;
	//解凍コマンドの作成
	MakeCommonMeltCmd(&aCmdLine);
	aCmdLine += string("\"")+mArcFName+"\" ";
	//解凍ファイルを指定
	p = oMeltFiles.begin();
	while ( p != oMeltFiles.end() ) {
		aCmdLine += string("\"")+(*p)+"\" ";
		p++;
	}
	aCmdLine += string("-d \"")+oDestFolder+"\"";
	//解凍
	DBEXP("aCmdLine - TTransZipArchiver::Melt",aCmdLine);
	system(aCmdLine.c_str());
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::MeltAll(const char *oDestFolder)
{
	string		aCmdLine;

	if ( !IsSet() )
		return B_NO_INIT;
	//解凍コマンドの作成
	MakeCommonMeltCmd(&aCmdLine);
	aCmdLine += string("\"")+mArcFName+"\" ";
	aCmdLine += string("-d \"")+oDestFolder+"\" ";
	//解凍
	system(aCmdLine.c_str());
	return B_OK;
}
//------------------------------------------------------------------------------
const translation_format*	TTransZipArchiver::OutputFormats(int32 *oOutCount) const
		//書き出せる形式
{
	static translation_format	saOutputFmts[] = {
			{ TRANS_ZIP_ARCHIVER_TGT_TYPE,TRANS_ARCHIVER_RAW_TYPE,TRANS_ZIP_ARCHIVER_QUALITY,TRANS_ZIP_ARCHIVER_CAPACITY,TRANS_ZIP_ARCHIVER_TGT_MIME,TRANS_ZIP_ARCHIVER_TGT_NAME },	//ネイティブ
			{ B_TRANSLATOR_BITMAP,B_TRANSLATOR_BITMAP,TRANS_ZIP_ARCHIVER_QUALITY,TRANS_ZIP_ARCHIVER_CAPACITY,BITMAP_MIME,BITMAP_NAME },	//画像も扱える
	};
	DBEXP("TTransZipArchiver::OutputFormats","");
	*oOutCount = 2;
	return saOutputFmts;
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::SetTo(const entry_ref *oArcRef,uint32 oOpenMode)
{
	status_t	aResult;
	BPath		aPath;

	aResult = SUPER::SetTo(oArcRef,oOpenMode);
	if ( aResult != B_OK )
		return aResult;
	aPath.SetTo(oArcRef);
	strcpy(mArcFName,aPath.Path());
	return B_OK;
}
//------------------------------------------------------------------------------
int32	TTransZipArchiver::SubClassCompatiIdent() const
{
	return COMPATIBILITY_IDENT;
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::Translate(BPositionIO *oInSource,const translator_info *oInInfo,BMessage *oIoExtension
		,uint32 oOutType,BPositionIO *oOutDestination)
{
	status_t	aResult;
	
//	DBEXP("TTransZipArchiver::Translate",system("BeCopy /boot/home/WorkData/MyTga /boot/home"));
	switch ( oOutType ) {
	case TRANS_ZIP_ARCHIVER_TGT_TYPE:	//エンコード
		aResult = TranslateBitsToZip(oInSource,oInInfo,oIoExtension,oOutDestination);
		break;
	case 0:	//デフォルト
	case B_TRANSLATOR_BITMAP:	//デコード
		aResult = TranslateZipToBits(oInSource,oInInfo,oIoExtension,oOutDestination);
		break;
	default:
		aResult = B_NO_TRANSLATOR;
	}
	return aResult;
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::TranslateBitsToZip(BPositionIO *oInSource,const translator_info *oInInfo,BMessage *oIoExtension
		,BPositionIO *oOutDestination)
{
	char				aCmdLine[B_FILE_NAME_LENGTH*2];
	char				aVarDirName[B_FILE_NAME_LENGTH];	//ランダム（フルパス）
	char				aVarFName[B_FILE_NAME_LENGTH];		//aVarDirName+STORE_FNAME
	char				aVarZipFName[B_FILE_NAME_LENGTH];	//aVarFName+".zip"
	uint8*				aBuf;
	uint8*				aBitsHeaderPtr;
	off_t				aInSize;
	BFile				aVarFile;
	TranslatorBitmap	aBitsHeader;

	DBEXP("TTransZipArchiver::TranslateBitsToZip","--->");
	//ファイル・ディレクトリの生成
	GenerateTmpFName(aVarDirName);
	sprintf(aVarFName,"%s/%s",aVarDirName,STORE_FNAME);
	sprintf(aVarZipFName,"%s%s",aVarFName,FPERIEXT_ZIP);
//	DBEXP("TTransZipArchiver::TranslateBitsToZip - aVarZipFName",aVarZipFName);
	sprintf(aCmdLine,"mkdir %s",aVarDirName);
	system(aCmdLine);
	//入力サイズ
	oInSource->Seek(0,SEEK_END);
	aInSize = oInSource->Position();
	aBuf = new uint8 [aInSize];
	//ヘッダを読み込む（aBitsHeader.magicはエンディアン変換しない）
	aBitsHeaderPtr = (uint8 *)&aBitsHeader;
	oInSource->Seek(0,SEEK_SET);
	oInSource->Read(aBitsHeaderPtr,sizeof(aBitsHeader));
	aBitsHeader.bounds.left = B_BENDIAN_TO_HOST_FLOAT(aBitsHeader.bounds.left);
	aBitsHeader.bounds.right = B_BENDIAN_TO_HOST_FLOAT(aBitsHeader.bounds.right);
	aBitsHeader.bounds.top = B_BENDIAN_TO_HOST_FLOAT(aBitsHeader.bounds.top);
	aBitsHeader.bounds.bottom = B_BENDIAN_TO_HOST_FLOAT(aBitsHeader.bounds.bottom);
	aBitsHeader.rowBytes = B_BENDIAN_TO_HOST_INT32(aBitsHeader.rowBytes);
	aBitsHeader.colors = (color_space)B_BENDIAN_TO_HOST_INT32(aBitsHeader.colors);
	aBitsHeader.dataSize = B_BENDIAN_TO_HOST_INT32(aBitsHeader.dataSize);
	//一時ファイルにビッツを書き出す
	aVarFile.SetTo(aVarFName,B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
	aVarFile.Write(aBitsHeaderPtr,sizeof(aBitsHeader));	//ヘッダの書き出し
	aInSize = oInSource->Read(aBuf,aInSize);
	aVarFile.Write(aBuf,aInSize);	//内容の書きだし
	aVarFile.Unset();
	delete aBuf;
	//一時ファイルを圧縮
	sprintf(aCmdLine,"zip -9 -j %s %s",aVarZipFName,aVarFName);
	if ( system(aCmdLine) != 0 )
		return B_NO_TRANSLATOR;
	//圧縮ファイルサイズ
	aVarFile.SetTo(aVarZipFName,B_READ_ONLY);
	aVarFile.GetSize(&aInSize);
	aBuf = new uint8 [aInSize];
	//一時ファイルからアーカイブの内容を読み込んで目的バッファにかきだし
	aVarFile.Read(aBuf,aInSize);
	oOutDestination->Seek(0,SEEK_SET);
	oOutDestination->Write(aBuf,aInSize);
	delete aBuf;
	//一時ファイルを削除
	sprintf(aCmdLine,"rm %s/*",aVarDirName);
	system(aCmdLine);
	sprintf(aCmdLine,"rmdir %s",aVarDirName);
	system(aCmdLine);
	return B_OK;
}
//------------------------------------------------------------------------------
status_t	TTransZipArchiver::TranslateZipToBits(BPositionIO *oInSource,const translator_info *oInInfo,BMessage *oIoExtension
		,BPositionIO *oOutDestination)
{
	char				aCmdLine[B_FILE_NAME_LENGTH*2];
	char				aVarMeltedDirName[B_FILE_NAME_LENGTH];	//解凍データを吐き出すフォルダ：ランダム（フルパス）
	char				aVarZipDirName[B_FILE_NAME_LENGTH];		//ソースを吐き出すフォルダ：ランダム（フルパス）
	char				aVarZipFName[B_FILE_NAME_LENGTH];		//aVarZipDirName+STORE_FNAME+".zip"
	float				aBmpHeight,aBmpWidth;
	uint8*				aBuf;
	uint8*				aBitsHeaderPtr;
	entry_ref			aMeltedERef;
	off_t				aInSize;
	BDirectory			aVarDir;
	BFile				aVarFile;
	BPath				aVarDirPath;
	TranslatorBitmap	aBitsHeader;

	DBEXP("TTransZipArchiver::TranslateZipToBits","<---");
	//一時ファイル名の生成
	GenerateTmpFName(aVarMeltedDirName);
	GenerateTmpFName(aVarZipDirName);
	sprintf(aVarZipFName,"%s/%s%s",aVarZipDirName,STORE_FNAME,FPERIEXT_ZIP);
	sprintf(aCmdLine,"mkdir %s",aVarZipDirName);
	system(aCmdLine);
	//入力サイズ
	oInSource->Seek(0,SEEK_END);
	aInSize = oInSource->Position();
	aBuf = new uint8 [aInSize];
	//一時ファイルにＺＩＰを書き出す
	aVarFile.SetTo(aVarZipFName,B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
	oInSource->Seek(0,SEEK_SET);
	oInSource->Read(aBuf,aInSize);
	aVarFile.Write(aBuf,aInSize);
	aVarFile.Unset();
	delete aBuf;
	//一時ファイルを解凍
	sprintf(aCmdLine,"unzip %s -d %s",aVarZipFName,aVarMeltedDirName);
	if ( system(aCmdLine) != 0 )
		return B_NO_TRANSLATOR;
	//解凍ファイルを得る
	aVarDir.SetTo(aVarMeltedDirName);
	if ( aVarDir.GetNextRef(&aMeltedERef) != B_OK )
		return B_NO_TRANSLATOR;
	//解凍ファイルサイズ
	aVarFile.SetTo(&aMeltedERef,B_READ_ONLY);
	aVarFile.GetSize(&aInSize);
	aBuf = new uint8 [aInSize];
	//解凍ファイルからビッツヘッダを読み込む（aBitsHeader.magicはエンディアン変換しない）
	aBitsHeaderPtr = (uint8 *)&aBitsHeader;
	aVarFile.Read(aBitsHeaderPtr,sizeof(aBitsHeader));
	aBmpWidth = aBitsHeader.bounds.right-aBitsHeader.bounds.left;
	aBmpHeight = aBitsHeader.bounds.bottom-aBitsHeader.bounds.top;
	aBitsHeader.bounds.left = B_HOST_TO_BENDIAN_FLOAT(0);
	aBitsHeader.bounds.right = B_HOST_TO_BENDIAN_FLOAT(aBmpWidth);
	aBitsHeader.bounds.top = B_HOST_TO_BENDIAN_FLOAT(0);
	aBitsHeader.bounds.bottom = B_HOST_TO_BENDIAN_FLOAT(aBmpHeight);
	aBitsHeader.rowBytes = B_HOST_TO_BENDIAN_INT32(aBitsHeader.rowBytes);
	aBitsHeader.colors = (color_space)B_HOST_TO_BENDIAN_INT32(aBitsHeader.colors);
	aBitsHeader.dataSize = B_HOST_TO_BENDIAN_INT32(aBitsHeader.dataSize);
	oOutDestination->Seek(0,SEEK_SET);
	oOutDestination->Write(aBitsHeaderPtr,sizeof(aBitsHeader));
	oOutDestination->Seek(sizeof(aBitsHeader),SEEK_SET);	//ないとダメ
	//解凍ファイルの内容を読み込んで目的バッファにかきだし
	aInSize = aVarFile.Read(aBuf,aInSize);
	oOutDestination->Write(aBuf,aInSize);
	delete aBuf;
	//一時ファイルを削除
	sprintf(aCmdLine,"rm %s/*",aVarZipDirName);
	system(aCmdLine);
	sprintf(aCmdLine,"rmdir %s",aVarZipDirName);
	system(aCmdLine);
	sprintf(aCmdLine,"rm %s/*",aVarMeltedDirName);
	system(aCmdLine);
	sprintf(aCmdLine,"rmdir %s",aVarMeltedDirName);
	system(aCmdLine);
	return B_OK;
}
//------------------------------------------------------------------------------
const char*	TTransZipArchiver::TranslatorInfo() const
{
	static char	saInfo[] = "Trans Zip Archiver by ＳＨＩＮＴＡ";

	return saInfo;
}
//------------------------------------------------------------------------------
const char*	TTransZipArchiver::TranslatorName() const
{
	static char	saName[] = "Trans Zip Archiver";

	return saName;
}
//------------------------------------------------------------------------------
int32		TTransZipArchiver::TranslatorVersion() const
{
	return 100;
}
//------------------------------------------------------------------------------
void	TTransZipArchiver::Unset()
{
	SUPER::Unset();
	mArcFName[0] = '\0';
}
//==============================================================================
BTranslator*	make_nth_translator(int32 oN,image_id oYou,uint32 oFlags, ...)
{
	if ( oN == 0 )
		return new TTransZipArchiver();
	else
		return NULL;
}
//------------------------------------------------------------------------------
#if 0
TTransZipArchiver*	gTranslator = NULL;
BLocker				gLocker;
//------------------------------------------------------------------------------
BTranslator*	make_nth_translator(int32 oN,image_id oYou,uint32 oFlags, ...)
{
	TTransZipArchiver*	aResult;
	
	gLocker.Lock();
	if ( gTranslator == NULL ) {
		aResult = gTranslator = new TTransZipArchiver();
	} else {
		aResult = NULL;
	}
	gLocker.Unlock();
	return aResult;
}
#endif
//------------------------------------------------------------------------------
#if 0
BTranslator*	make_nth_translator(int32 oN,image_id oYou,uint32 oFlags, ...)
{
	static int32				saMadeIdx = -1;
	static BLocker				saLocker;
//	static TTransZipArchiver*	saTranslator = NULL;

	TTransZipArchiver*	aResult;

	if ( oN == 0 ) {
		aResult = new TTransZipArchiver();
	} else {
		aResult = NULL;
	}
//	saLocker.Lock();
//	if ( oN > saMadeIdx ) {
//		DBEXP("番目 - make_nth_translator",oN);
//		saMadeIdx = oN;
//		aResult = new TTransZipArchiver();
//	} else {
//		aResult = NULL;
//	}
//	if ( saTranslator == NULL ) {
//		aResult = saTranslator = new TTransZipArchiver();
//	} else {
//		aResult = NULL;
//	}
//	saLocker.Unlock();
	return aResult;
}
#endif
//==============================================================================
